from enum import Enum


class _NiceEnum(Enum):
    def __str__(self):
        return self.value
    
    def __repr__(self):
        return self.value


class CURRENCIES:
    class PAXGTMN(_NiceEnum):
        symbol = 'PAXGTMN'
        baseAsset = 'PAXG'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'پکسوس گلد - تومان'
        faBaseAsset = 'پکسوس گلد'
        faQuoteAsset = 'تومان'
        stepSize = 6
        tickSize = 0
        minQty = 1e-06
        minNotional = 100000

    class PAXGUSDT(_NiceEnum):
        symbol = 'PAXGUSDT'
        baseAsset = 'PAXG'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'پکسوس گلد - تتر'
        faBaseAsset = 'پکسوس گلد'
        faQuoteAsset = 'تتر'
        stepSize = 6
        tickSize = 2
        minQty = 1e-06
        minNotional = 10

    class PAXGBTC(_NiceEnum):
        symbol = 'PAXGBTC'
        baseAsset = 'PAXG'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'پکسوس گلد - بیت کوین'
        faBaseAsset = 'پکسوس گلد'
        faQuoteAsset = 'بیت کوین'
        stepSize = 4
        tickSize = 5
        minQty = 0.0001
        minNotional = 0.0001

    class ONEINCHTMN(_NiceEnum):
        symbol = '1INCHTMN'
        baseAsset = '1INCH'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'وان اینچ - تومان'
        faBaseAsset = 'وان اینچ'
        faQuoteAsset = 'تومان'
        stepSize = 2
        tickSize = 0
        minQty = 0.01
        minNotional = 100000

    class ONEINCHUSDT(_NiceEnum):
        symbol = '1INCHUSDT'
        baseAsset = '1INCH'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'وان اینچ - تتر'
        faBaseAsset = 'وان اینچ'
        faQuoteAsset = 'تتر'
        stepSize = 1
        tickSize = 3
        minQty = 0.1
        minNotional = 10

    class ONEINCHBTC(_NiceEnum):
        symbol = '1INCHBTC'
        baseAsset = '1INCH'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'وان اینچ - بیت کوین'
        faBaseAsset = 'وان اینچ'
        faQuoteAsset = 'بیت کوین'
        stepSize = 1
        tickSize = 7
        minQty = 0.1
        minNotional = 0.0001

    class YFITMN(_NiceEnum):
        symbol = 'YFITMN'
        baseAsset = 'YFI'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'یرن فایننس - تومان'
        faBaseAsset = 'یرن فایننس'
        faQuoteAsset = 'تومان'
        stepSize = 5
        tickSize = 0
        minQty = 1e-05
        minNotional = 100000

    class YFIUSDT(_NiceEnum):
        symbol = 'YFIUSDT'
        baseAsset = 'YFI'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'یرن فایننس - تتر'
        faBaseAsset = 'یرن فایننس'
        faQuoteAsset = 'تتر'
        stepSize = 5
        tickSize = 2
        minQty = 1e-05
        minNotional = 10

    class YFIBTC(_NiceEnum):
        symbol = 'YFIBTC'
        baseAsset = 'YFI'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'یرن فایننس - بیت کوین'
        faBaseAsset = 'یرن فایننس'
        faQuoteAsset = 'بیت کوین'
        stepSize = 2
        tickSize = 6
        minQty = 0.01
        minNotional = 0.0001

    class CELRTMN(_NiceEnum):
        symbol = 'CELRTMN'
        baseAsset = 'CELR'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'سلر - تومان'
        faBaseAsset = 'سلر'
        faQuoteAsset = 'تومان'
        stepSize = 1
        tickSize = 0
        minQty = 0.1
        minNotional = 100000

    class CELRUSDT(_NiceEnum):
        symbol = 'CELRUSDT'
        baseAsset = 'CELR'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'سلر - تتر'
        faBaseAsset = 'سلر'
        faQuoteAsset = 'تتر'
        stepSize = 1
        tickSize = 5
        minQty = 0.1
        minNotional = 10

    class CELRBTC(_NiceEnum):
        symbol = 'CELRBTC'
        baseAsset = 'CELR'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'سلر - بیت کوین'
        faBaseAsset = 'سلر'
        faQuoteAsset = 'بیت کوین'
        stepSize = 0
        tickSize = 8
        minQty = 1
        minNotional = 0.0001

    class APETMN(_NiceEnum):
        symbol = 'APETMN'
        baseAsset = 'APE'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'ایپ کوین - تومان'
        faBaseAsset = 'ایپ کوین'
        faQuoteAsset = 'تومان'
        stepSize = 2
        tickSize = 0
        minQty = 0.01
        minNotional = 100000

    class APEUSDT(_NiceEnum):
        symbol = 'APEUSDT'
        baseAsset = 'APE'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'ایپ کوین - تتر'
        faBaseAsset = 'ایپ کوین'
        faQuoteAsset = 'تتر'
        stepSize = 2
        tickSize = 3
        minQty = 0.01
        minNotional = 10

    class APEBTC(_NiceEnum):
        symbol = 'APEBTC'
        baseAsset = 'APE'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'ایپ کوین - بیت کوین'
        faBaseAsset = 'ایپ کوین'
        faQuoteAsset = 'بیت کوین'
        stepSize = 1
        tickSize = 7
        minQty = 0.1
        minNotional = 0.0001

    class POLSTMN(_NiceEnum):
        symbol = 'POLSTMN'
        baseAsset = 'POLS'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'پولکا استارتر - تومان'
        faBaseAsset = 'پولکا استارتر'
        faQuoteAsset = 'تومان'
        stepSize = 1
        tickSize = 0
        minQty = 0.1
        minNotional = 100000

    class POLSUSDT(_NiceEnum):
        symbol = 'POLSUSDT'
        baseAsset = 'POLS'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'پولکا استارتر - تتر'
        faBaseAsset = 'پولکا استارتر'
        faQuoteAsset = 'تتر'
        stepSize = 1
        tickSize = 5
        minQty = 0.1
        minNotional = 10

    class POLSBTC(_NiceEnum):
        symbol = 'POLSBTC'
        baseAsset = 'POLS'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'پولکا استارتر - بیت کوین'
        faBaseAsset = 'پولکا استارتر'
        faQuoteAsset = 'بیت کوین'
        stepSize = 0
        tickSize = 8
        minQty = 1
        minNotional = 0.0001

    class DAITMN(_NiceEnum):
        symbol = 'DAITMN'
        baseAsset = 'DAI'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'دای - تومان'
        faBaseAsset = 'دای'
        faQuoteAsset = 'تومان'
        stepSize = 2
        tickSize = 0
        minQty = 0.01
        minNotional = 100000

    class AAVETMN(_NiceEnum):
        symbol = 'AAVETMN'
        baseAsset = 'AAVE'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'آوه - تومان'
        faBaseAsset = 'آوه'
        faQuoteAsset = 'تومان'
        stepSize = 5
        tickSize = 0
        minQty = 1e-05
        minNotional = 100000

    class AAVEUSDT(_NiceEnum):
        symbol = 'AAVEUSDT'
        baseAsset = 'AAVE'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'آوه - تتر'
        faBaseAsset = 'آوه'
        faQuoteAsset = 'تتر'
        stepSize = 5
        tickSize = 2
        minQty = 1e-05
        minNotional = 10

    class AAVEBTC(_NiceEnum):
        symbol = 'AAVEBTC'
        baseAsset = 'AAVE'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'آوه - بیت کوین'
        faBaseAsset = 'آوه'
        faQuoteAsset = 'بیت کوین'
        stepSize = 2
        tickSize = 6
        minQty = 0.01
        minNotional = 0.0001

    class GALATMN(_NiceEnum):
        symbol = 'GALATMN'
        baseAsset = 'GALA'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'گالا - تومان'
        faBaseAsset = 'گالا'
        faQuoteAsset = 'تومان'
        stepSize = 1
        tickSize = 0
        minQty = 0.1
        minNotional = 100000

    class GALAUSDT(_NiceEnum):
        symbol = 'GALAUSDT'
        baseAsset = 'GALA'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'گالا - تتر'
        faBaseAsset = 'گالا'
        faQuoteAsset = 'تتر'
        stepSize = 1
        tickSize = 5
        minQty = 0.1
        minNotional = 10

    class GALABTC(_NiceEnum):
        symbol = 'GALABTC'
        baseAsset = 'GALA'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'گالا - بیت کوین'
        faBaseAsset = 'گالا'
        faQuoteAsset = 'بیت کوین'
        stepSize = 0
        tickSize = 8
        minQty = 1
        minNotional = 0.0001

    class ETCTMN(_NiceEnum):
        symbol = 'ETCTMN'
        baseAsset = 'ETC'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'اتریوم کلاسیک - تومان'
        faBaseAsset = 'اتریوم کلاسیک'
        faQuoteAsset = 'تومان'
        stepSize = 5
        tickSize = 0
        minQty = 1e-05
        minNotional = 100000

    class ETCUSDT(_NiceEnum):
        symbol = 'ETCUSDT'
        baseAsset = 'ETC'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'اتریوم کلاسیک - تتر'
        faBaseAsset = 'اتریوم کلاسیک'
        faQuoteAsset = 'تتر'
        stepSize = 5
        tickSize = 2
        minQty = 1e-05
        minNotional = 10

    class ETCBTC(_NiceEnum):
        symbol = 'ETCBTC'
        baseAsset = 'ETC'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'اتریوم کلاسیک - بیت کوین'
        faBaseAsset = 'اتریوم کلاسیک'
        faQuoteAsset = 'بیت کوین'
        stepSize = 2
        tickSize = 6
        minQty = 0.01
        minNotional = 0.0001

    class MKRTMN(_NiceEnum):
        symbol = 'MKRTMN'
        baseAsset = 'MKR'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'میکر - تومان'
        faBaseAsset = 'میکر'
        faQuoteAsset = 'تومان'
        stepSize = 5
        tickSize = 0
        minQty = 1e-05
        minNotional = 100000

    class MKRUSDT(_NiceEnum):
        symbol = 'MKRUSDT'
        baseAsset = 'MKR'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'میکر - تتر'
        faBaseAsset = 'میکر'
        faQuoteAsset = 'تتر'
        stepSize = 4
        tickSize = 2
        minQty = 0.0001
        minNotional = 10

    class MKRBTC(_NiceEnum):
        symbol = 'MKRBTC'
        baseAsset = 'MKR'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'میکر - بیت کوین'
        faBaseAsset = 'میکر'
        faQuoteAsset = 'بیت کوین'
        stepSize = 4
        tickSize = 5
        minQty = 0.0001
        minNotional = 0.0001

    class BTCTMN(_NiceEnum):
        symbol = 'BTCTMN'
        baseAsset = 'BTC'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'بیت کوین - تومان'
        faBaseAsset = 'بیت کوین'
        faQuoteAsset = 'تومان'
        stepSize = 6
        tickSize = 0
        minQty = 1e-06
        minNotional = 100000

    class BTCUSDT(_NiceEnum):
        symbol = 'BTCUSDT'
        baseAsset = 'BTC'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'بیت کوین - تتر'
        faBaseAsset = 'بیت کوین'
        faQuoteAsset = 'تتر'
        stepSize = 6
        tickSize = 2
        minQty = 1e-06
        minNotional = 10

    class ETHTMN(_NiceEnum):
        symbol = 'ETHTMN'
        baseAsset = 'ETH'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'اتریوم - تومان'
        faBaseAsset = 'اتریوم'
        faQuoteAsset = 'تومان'
        stepSize = 5
        tickSize = 0
        minQty = 1e-05
        minNotional = 100000

    class ETHUSDT(_NiceEnum):
        symbol = 'ETHUSDT'
        baseAsset = 'ETH'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'اتریوم - تتر'
        faBaseAsset = 'اتریوم'
        faQuoteAsset = 'تتر'
        stepSize = 5
        tickSize = 2
        minQty = 1e-05
        minNotional = 10

    class ETHBTC(_NiceEnum):
        symbol = 'ETHBTC'
        baseAsset = 'ETH'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'اتریوم - بیت کوین'
        faBaseAsset = 'اتریوم'
        faQuoteAsset = 'بیت کوین'
        stepSize = 3
        tickSize = 6
        minQty = 0.001
        minNotional = 0.0001

    class DOGETMN(_NiceEnum):
        symbol = 'DOGETMN'
        baseAsset = 'DOGE'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'دوج کوین - تومان'
        faBaseAsset = 'دوج کوین'
        faQuoteAsset = 'تومان'
        stepSize = 1
        tickSize = 0
        minQty = 0.1
        minNotional = 100000

    class DOGEUSDT(_NiceEnum):
        symbol = 'DOGEUSDT'
        baseAsset = 'DOGE'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'دوج کوین - تتر'
        faBaseAsset = 'دوج کوین'
        faQuoteAsset = 'تتر'
        stepSize = 1
        tickSize = 5
        minQty = 0.1
        minNotional = 10

    class DOGEBTC(_NiceEnum):
        symbol = 'DOGEBTC'
        baseAsset = 'DOGE'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'دوج کوین - بیت کوین'
        faBaseAsset = 'دوج کوین'
        faQuoteAsset = 'بیت کوین'
        stepSize = 0
        tickSize = 8
        minQty = 1
        minNotional = 0.0001

    class BCHTMN(_NiceEnum):
        symbol = 'BCHTMN'
        baseAsset = 'BCH'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'بیت کوین کش - تومان'
        faBaseAsset = 'بیت کوین کش'
        faQuoteAsset = 'تومان'
        stepSize = 5
        tickSize = 0
        minQty = 1e-05
        minNotional = 100000

    class BCHUSDT(_NiceEnum):
        symbol = 'BCHUSDT'
        baseAsset = 'BCH'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'بیت کوین کش - تتر'
        faBaseAsset = 'بیت کوین کش'
        faQuoteAsset = 'تتر'
        stepSize = 5
        tickSize = 2
        minQty = 1e-05
        minNotional = 10

    class BCHBTC(_NiceEnum):
        symbol = 'BCHBTC'
        baseAsset = 'BCH'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'بیت کوین کش - بیت کوین'
        faBaseAsset = 'بیت کوین کش'
        faQuoteAsset = 'بیت کوین'
        stepSize = 3
        tickSize = 6
        minQty = 0.001
        minNotional = 0.0001

    class LTCTMN(_NiceEnum):
        symbol = 'LTCTMN'
        baseAsset = 'LTC'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'لایت کوین - تومان'
        faBaseAsset = 'لایت کوین'
        faQuoteAsset = 'تومان'
        stepSize = 5
        tickSize = 0
        minQty = 1e-05
        minNotional = 100000

    class LTCUSDT(_NiceEnum):
        symbol = 'LTCUSDT'
        baseAsset = 'LTC'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'لایت کوین - تتر'
        faBaseAsset = 'لایت کوین'
        faQuoteAsset = 'تتر'
        stepSize = 5
        tickSize = 2
        minQty = 1e-05
        minNotional = 10

    class LTCBTC(_NiceEnum):
        symbol = 'LTCBTC'
        baseAsset = 'LTC'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'لایت کوین - بیت کوین'
        faBaseAsset = 'لایت کوین'
        faQuoteAsset = 'بیت کوین'
        stepSize = 2
        tickSize = 6
        minQty = 0.01
        minNotional = 0.0001

    class DASHTMN(_NiceEnum):
        symbol = 'DASHTMN'
        baseAsset = 'DASH'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'دش - تومان'
        faBaseAsset = 'دش'
        faQuoteAsset = 'تومان'
        stepSize = 5
        tickSize = 0
        minQty = 1e-05
        minNotional = 100000

    class DASHUSDT(_NiceEnum):
        symbol = 'DASHUSDT'
        baseAsset = 'DASH'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'دش - تتر'
        faBaseAsset = 'دش'
        faQuoteAsset = 'تتر'
        stepSize = 5
        tickSize = 2
        minQty = 1e-05
        minNotional = 10

    class DASHBTC(_NiceEnum):
        symbol = 'DASHBTC'
        baseAsset = 'DASH'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'دش - بیت کوین'
        faBaseAsset = 'دش'
        faQuoteAsset = 'بیت کوین'
        stepSize = 3
        tickSize = 6
        minQty = 0.001
        minNotional = 0.0001

    class USDTTMN(_NiceEnum):
        symbol = 'USDTTMN'
        baseAsset = 'USDT'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'تتر - تومان'
        faBaseAsset = 'تتر'
        faQuoteAsset = 'تومان'
        stepSize = 2
        tickSize = 0
        minQty = 0.01
        minNotional = 100000

    class XRPTMN(_NiceEnum):
        symbol = 'XRPTMN'
        baseAsset = 'XRP'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'ریپل - تومان'
        faBaseAsset = 'ریپل'
        faQuoteAsset = 'تومان'
        stepSize = 1
        tickSize = 0
        minQty = 0.1
        minNotional = 100000

    class XRPUSDT(_NiceEnum):
        symbol = 'XRPUSDT'
        baseAsset = 'XRP'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'ریپل - تتر'
        faBaseAsset = 'ریپل'
        faQuoteAsset = 'تتر'
        stepSize = 1
        tickSize = 5
        minQty = 0.1
        minNotional = 10

    class XRPBTC(_NiceEnum):
        symbol = 'XRPBTC'
        baseAsset = 'XRP'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'ریپل - بیت کوین'
        faBaseAsset = 'ریپل'
        faQuoteAsset = 'بیت کوین'
        stepSize = 0
        tickSize = 8
        minQty = 1
        minNotional = 0.0001

    class XLMTMN(_NiceEnum):
        symbol = 'XLMTMN'
        baseAsset = 'XLM'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'استلار - تومان'
        faBaseAsset = 'استلار'
        faQuoteAsset = 'تومان'
        stepSize = 1
        tickSize = 0
        minQty = 0.1
        minNotional = 100000

    class XLMUSDT(_NiceEnum):
        symbol = 'XLMUSDT'
        baseAsset = 'XLM'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'استلار - تتر'
        faBaseAsset = 'استلار'
        faQuoteAsset = 'تتر'
        stepSize = 1
        tickSize = 5
        minQty = 0.1
        minNotional = 10

    class XLMBTC(_NiceEnum):
        symbol = 'XLMBTC'
        baseAsset = 'XLM'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'استلار - بیت کوین'
        faBaseAsset = 'استلار'
        faQuoteAsset = 'بیت کوین'
        stepSize = 0
        tickSize = 8
        minQty = 1
        minNotional = 0.0001

    class EOSTMN(_NiceEnum):
        symbol = 'EOSTMN'
        baseAsset = 'EOS'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'ایاس - تومان'
        faBaseAsset = 'ایاس'
        faQuoteAsset = 'تومان'
        stepSize = 2
        tickSize = 0
        minQty = 0.01
        minNotional = 100000

    class EOSUSDT(_NiceEnum):
        symbol = 'EOSUSDT'
        baseAsset = 'EOS'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'ایاس - تتر'
        faBaseAsset = 'ایاس'
        faQuoteAsset = 'تتر'
        stepSize = 2
        tickSize = 4
        minQty = 0.01
        minNotional = 10

    class EOSBTC(_NiceEnum):
        symbol = 'EOSBTC'
        baseAsset = 'EOS'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'ایاس - بیت کوین'
        faBaseAsset = 'ایاس'
        faQuoteAsset = 'بیت کوین'
        stepSize = 2
        tickSize = 7
        minQty = 0.01
        minNotional = 0.0001

    class TRXTMN(_NiceEnum):
        symbol = 'TRXTMN'
        baseAsset = 'TRX'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'ترون - تومان'
        faBaseAsset = 'ترون'
        faQuoteAsset = 'تومان'
        stepSize = 1
        tickSize = 0
        minQty = 0.1
        minNotional = 100000

    class TRXUSDT(_NiceEnum):
        symbol = 'TRXUSDT'
        baseAsset = 'TRX'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'ترون - تتر'
        faBaseAsset = 'ترون'
        faQuoteAsset = 'تتر'
        stepSize = 1
        tickSize = 5
        minQty = 0.1
        minNotional = 10

    class TRXBTC(_NiceEnum):
        symbol = 'TRXBTC'
        baseAsset = 'TRX'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'ترون - بیت کوین'
        faBaseAsset = 'ترون'
        faQuoteAsset = 'بیت کوین'
        stepSize = 0
        tickSize = 8
        minQty = 1
        minNotional = 0.0001

    class ADATMN(_NiceEnum):
        symbol = 'ADATMN'
        baseAsset = 'ADA'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'کاردانو - تومان'
        faBaseAsset = 'کاردانو'
        faQuoteAsset = 'تومان'
        stepSize = 1
        tickSize = 0
        minQty = 0.1
        minNotional = 100000

    class ADAUSDT(_NiceEnum):
        symbol = 'ADAUSDT'
        baseAsset = 'ADA'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'کاردانو - تتر'
        faBaseAsset = 'کاردانو'
        faQuoteAsset = 'تتر'
        stepSize = 1
        tickSize = 3
        minQty = 0.1
        minNotional = 10

    class ADABTC(_NiceEnum):
        symbol = 'ADABTC'
        baseAsset = 'ADA'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'کاردانو - بیت کوین'
        faBaseAsset = 'کاردانو'
        faQuoteAsset = 'بیت کوین'
        stepSize = 1
        tickSize = 7
        minQty = 0.1
        minNotional = 0.0001

    class BNBTMN(_NiceEnum):
        symbol = 'BNBTMN'
        baseAsset = 'BNB'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'بایننس کوین - تومان'
        faBaseAsset = 'بایننس کوین'
        faQuoteAsset = 'تومان'
        stepSize = 3
        tickSize = 0
        minQty = 0.001
        minNotional = 100000

    class BNBUSDT(_NiceEnum):
        symbol = 'BNBUSDT'
        baseAsset = 'BNB'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'بایننس کوین - تتر'
        faBaseAsset = 'بایننس کوین'
        faQuoteAsset = 'تتر'
        stepSize = 3
        tickSize = 4
        minQty = 0.001
        minNotional = 10

    class BNBBTC(_NiceEnum):
        symbol = 'BNBBTC'
        baseAsset = 'BNB'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'بایننس کوین - بیت کوین'
        faBaseAsset = 'بایننس کوین'
        faQuoteAsset = 'بیت کوین'
        stepSize = 3
        tickSize = 5
        minQty = 0.001
        minNotional = 0.0001

    class ATOMTMN(_NiceEnum):
        symbol = 'ATOMTMN'
        baseAsset = 'ATOM'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'اتم - تومان'
        faBaseAsset = 'اتم'
        faQuoteAsset = 'تومان'
        stepSize = 2
        tickSize = 0
        minQty = 0.01
        minNotional = 100000

    class ATOMUSDT(_NiceEnum):
        symbol = 'ATOMUSDT'
        baseAsset = 'ATOM'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'اتم - تتر'
        faBaseAsset = 'اتم'
        faQuoteAsset = 'تتر'
        stepSize = 2
        tickSize = 2
        minQty = 0.01
        minNotional = 10

    class ATOMBTC(_NiceEnum):
        symbol = 'ATOMBTC'
        baseAsset = 'ATOM'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'اتم - بیت کوین'
        faBaseAsset = 'اتم'
        faQuoteAsset = 'بیت کوین'
        stepSize = 2
        tickSize = 6
        minQty = 0.01
        minNotional = 0.0001

    class MATICTMN(_NiceEnum):
        symbol = 'MATICTMN'
        baseAsset = 'MATIC'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'متیک - تومان'
        faBaseAsset = 'متیک'
        faQuoteAsset = 'تومان'
        stepSize = 1
        tickSize = 0
        minQty = 0.1
        minNotional = 100000

    class MATICUSDT(_NiceEnum):
        symbol = 'MATICUSDT'
        baseAsset = 'MATIC'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'متیک - تتر'
        faBaseAsset = 'متیک'
        faQuoteAsset = 'تتر'
        stepSize = 1
        tickSize = 3
        minQty = 0.1
        minNotional = 10

    class MATICBTC(_NiceEnum):
        symbol = 'MATICBTC'
        baseAsset = 'MATIC'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'متیک - بیت کوین'
        faBaseAsset = 'متیک'
        faQuoteAsset = 'بیت کوین'
        stepSize = 1
        tickSize = 7
        minQty = 0.1
        minNotional = 0.0001

    class FTMTMN(_NiceEnum):
        symbol = 'FTMTMN'
        baseAsset = 'FTM'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'فانتوم - تومان'
        faBaseAsset = 'فانتوم'
        faQuoteAsset = 'تومان'
        stepSize = 2
        tickSize = 0
        minQty = 0.01
        minNotional = 100000

    class FTMUSDT(_NiceEnum):
        symbol = 'FTMUSDT'
        baseAsset = 'FTM'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'فانتوم - تتر'
        faBaseAsset = 'فانتوم'
        faQuoteAsset = 'تتر'
        stepSize = 2
        tickSize = 4
        minQty = 0.01
        minNotional = 10

    class FTMBTC(_NiceEnum):
        symbol = 'FTMBTC'
        baseAsset = 'FTM'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'فانتوم - بیت کوین'
        faBaseAsset = 'فانتوم'
        faQuoteAsset = 'بیت کوین'
        stepSize = 0
        tickSize = 8
        minQty = 1
        minNotional = 0.0001

    class DOTTMN(_NiceEnum):
        symbol = 'DOTTMN'
        baseAsset = 'DOT'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'پولکادات - تومان'
        faBaseAsset = 'پولکادات'
        faQuoteAsset = 'تومان'
        stepSize = 2
        tickSize = 0
        minQty = 0.01
        minNotional = 100000

    class DOTUSDT(_NiceEnum):
        symbol = 'DOTUSDT'
        baseAsset = 'DOT'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'پولکادات - تتر'
        faBaseAsset = 'پولکادات'
        faQuoteAsset = 'تتر'
        stepSize = 2
        tickSize = 3
        minQty = 0.01
        minNotional = 10

    class DOTBTC(_NiceEnum):
        symbol = 'DOTBTC'
        baseAsset = 'DOT'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'پولکادات - بیت کوین'
        faBaseAsset = 'پولکادات'
        faQuoteAsset = 'بیت کوین'
        stepSize = 1
        tickSize = 7
        minQty = 0.1
        minNotional = 0.0001

    class SHIBTMN(_NiceEnum):
        symbol = 'SHIBTMN'
        baseAsset = 'SHIB'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'شیبا - تومان'
        faBaseAsset = 'شیبا'
        faQuoteAsset = 'تومان'
        stepSize = 0
        tickSize = 3
        minQty = 1
        minNotional = 100000

    class SHIBUSDT(_NiceEnum):
        symbol = 'SHIBUSDT'
        baseAsset = 'SHIB'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'شیبا - تتر'
        faBaseAsset = 'شیبا'
        faQuoteAsset = 'تتر'
        stepSize = 0
        tickSize = 8
        minQty = 1
        minNotional = 10

    class FILTMN(_NiceEnum):
        symbol = 'FILTMN'
        baseAsset = 'FIL'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'فایل کوین - تومان'
        faBaseAsset = 'فایل کوین'
        faQuoteAsset = 'تومان'
        stepSize = 2
        tickSize = 0
        minQty = 0.01
        minNotional = 100000

    class FILUSDT(_NiceEnum):
        symbol = 'FILUSDT'
        baseAsset = 'FIL'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'فایل کوین - تتر'
        faBaseAsset = 'فایل کوین'
        faQuoteAsset = 'تتر'
        stepSize = 2
        tickSize = 2
        minQty = 0.01
        minNotional = 10

    class FILBTC(_NiceEnum):
        symbol = 'FILBTC'
        baseAsset = 'FIL'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'فایل کوین - بیت کوین'
        faBaseAsset = 'فایل کوین'
        faQuoteAsset = 'بیت کوین'
        stepSize = 2
        tickSize = 6
        minQty = 0.01
        minNotional = 0.0001

    class CAKETMN(_NiceEnum):
        symbol = 'CAKETMN'
        baseAsset = 'CAKE'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'پن کیک سوآپ - تومان'
        faBaseAsset = 'پن کیک سوآپ'
        faQuoteAsset = 'تومان'
        stepSize = 2
        tickSize = 0
        minQty = 0.01
        minNotional = 100000

    class CAKEUSDT(_NiceEnum):
        symbol = 'CAKEUSDT'
        baseAsset = 'CAKE'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'پن کیک سوآپ - تتر'
        faBaseAsset = 'پن کیک سوآپ'
        faQuoteAsset = 'تتر'
        stepSize = 2
        tickSize = 2
        minQty = 0.01
        minNotional = 10

    class CAKEBTC(_NiceEnum):
        symbol = 'CAKEBTC'
        baseAsset = 'CAKE'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'پن کیک سوآپ - بیت کوین'
        faBaseAsset = 'پن کیک سوآپ'
        faQuoteAsset = 'بیت کوین'
        stepSize = 2
        tickSize = 6
        minQty = 0.01
        minNotional = 0.0001

    class LINKTMN(_NiceEnum):
        symbol = 'LINKTMN'
        baseAsset = 'LINK'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'چین لینک - تومان'
        faBaseAsset = 'چین لینک'
        faQuoteAsset = 'تومان'
        stepSize = 2
        tickSize = 0
        minQty = 0.01
        minNotional = 100000

    class LINKUSDT(_NiceEnum):
        symbol = 'LINKUSDT'
        baseAsset = 'LINK'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'چین لینک - تتر'
        faBaseAsset = 'چین لینک'
        faQuoteAsset = 'تتر'
        stepSize = 2
        tickSize = 2
        minQty = 0.01
        minNotional = 10

    class LINKBTC(_NiceEnum):
        symbol = 'LINKBTC'
        baseAsset = 'LINK'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'چین لینک - بیت کوین'
        faBaseAsset = 'چین لینک'
        faQuoteAsset = 'بیت کوین'
        stepSize = 2
        tickSize = 6
        minQty = 0.01
        minNotional = 0.0001

    class UNITMN(_NiceEnum):
        symbol = 'UNITMN'
        baseAsset = 'UNI'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'یونی سوآپ - تومان'
        faBaseAsset = 'یونی سوآپ'
        faQuoteAsset = 'تومان'
        stepSize = 2
        tickSize = 0
        minQty = 0.01
        minNotional = 100000

    class UNIUSDT(_NiceEnum):
        symbol = 'UNIUSDT'
        baseAsset = 'UNI'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'یونی سوآپ - تتر'
        faBaseAsset = 'یونی سوآپ'
        faQuoteAsset = 'تتر'
        stepSize = 2
        tickSize = 2
        minQty = 0.01
        minNotional = 10

    class UNIBTC(_NiceEnum):
        symbol = 'UNIBTC'
        baseAsset = 'UNI'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'یونی سوآپ - بیت کوین'
        faBaseAsset = 'یونی سوآپ'
        faQuoteAsset = 'بیت کوین'
        stepSize = 2
        tickSize = 6
        minQty = 0.01
        minNotional = 0.0001

    class RUNETMN(_NiceEnum):
        symbol = 'RUNETMN'
        baseAsset = 'RUNE'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'تورچین - تومان'
        faBaseAsset = 'تورچین'
        faQuoteAsset = 'تومان'
        stepSize = 2
        tickSize = 0
        minQty = 0.01
        minNotional = 100000

    class RUNEUSDT(_NiceEnum):
        symbol = 'RUNEUSDT'
        baseAsset = 'RUNE'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'تورچین - تتر'
        faBaseAsset = 'تورچین'
        faQuoteAsset = 'تتر'
        stepSize = 2
        tickSize = 2
        minQty = 0.01
        minNotional = 10

    class RUNEBTC(_NiceEnum):
        symbol = 'RUNEBTC'
        baseAsset = 'RUNE'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'تورچین - بیت کوین'
        faBaseAsset = 'تورچین'
        faQuoteAsset = 'بیت کوین'
        stepSize = 2
        tickSize = 6
        minQty = 0.01
        minNotional = 0.0001

    class CHZTMN(_NiceEnum):
        symbol = 'CHZTMN'
        baseAsset = 'CHZ'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'چیلیز - تومان'
        faBaseAsset = 'چیلیز'
        faQuoteAsset = 'تومان'
        stepSize = 1
        tickSize = 0
        minQty = 0.1
        minNotional = 100000

    class CHZUSDT(_NiceEnum):
        symbol = 'CHZUSDT'
        baseAsset = 'CHZ'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'چیلیز - تتر'
        faBaseAsset = 'چیلیز'
        faQuoteAsset = 'تتر'
        stepSize = 1
        tickSize = 5
        minQty = 0.1
        minNotional = 10

    class CHZBTC(_NiceEnum):
        symbol = 'CHZBTC'
        baseAsset = 'CHZ'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'چیلیز - بیت کوین'
        faBaseAsset = 'چیلیز'
        faQuoteAsset = 'بیت کوین'
        stepSize = 0
        tickSize = 8
        minQty = 1
        minNotional = 0.0001

    class BTTCTMN(_NiceEnum):
        symbol = 'BTTCTMN'
        baseAsset = 'BTTC'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'بیت تورنت(جدید) - تومان'
        faBaseAsset = 'بیت تورنت(جدید)'
        faQuoteAsset = 'تومان'
        stepSize = 0
        tickSize = 6
        minQty = 1
        minNotional = 100000

    class BTTCUSDT(_NiceEnum):
        symbol = 'BTTCUSDT'
        baseAsset = 'BTTC'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'بیت تورنت(جدید) - تتر'
        faBaseAsset = 'بیت تورنت(جدید)'
        faQuoteAsset = 'تتر'
        stepSize = 0
        tickSize = 8
        minQty = 1
        minNotional = 10

    class MANATMN(_NiceEnum):
        symbol = 'MANATMN'
        baseAsset = 'MANA'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'مانا - تومان'
        faBaseAsset = 'مانا'
        faQuoteAsset = 'تومان'
        stepSize = 1
        tickSize = 0
        minQty = 0.1
        minNotional = 100000

    class MANAUSDT(_NiceEnum):
        symbol = 'MANAUSDT'
        baseAsset = 'MANA'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'مانا - تتر'
        faBaseAsset = 'مانا'
        faQuoteAsset = 'تتر'
        stepSize = 1
        tickSize = 3
        minQty = 0.1
        minNotional = 10

    class MANABTC(_NiceEnum):
        symbol = 'MANABTC'
        baseAsset = 'MANA'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'مانا - بیت کوین'
        faBaseAsset = 'مانا'
        faQuoteAsset = 'بیت کوین'
        stepSize = 1
        tickSize = 7
        minQty = 0.1
        minNotional = 0.0001

    class AXSTMN(_NiceEnum):
        symbol = 'AXSTMN'
        baseAsset = 'AXS'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'اکسی - تومان'
        faBaseAsset = 'اکسی'
        faQuoteAsset = 'تومان'
        stepSize = 5
        tickSize = 0
        minQty = 1e-05
        minNotional = 100000

    class AXSUSDT(_NiceEnum):
        symbol = 'AXSUSDT'
        baseAsset = 'AXS'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'اکسی - تتر'
        faBaseAsset = 'اکسی'
        faQuoteAsset = 'تتر'
        stepSize = 5
        tickSize = 2
        minQty = 1e-05
        minNotional = 10

    class AXSBTC(_NiceEnum):
        symbol = 'AXSBTC'
        baseAsset = 'AXS'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'اکسی - بیت کوین'
        faBaseAsset = 'اکسی'
        faQuoteAsset = 'بیت کوین'
        stepSize = 2
        tickSize = 6
        minQty = 0.01
        minNotional = 0.0001

    class SANDTMN(_NiceEnum):
        symbol = 'SANDTMN'
        baseAsset = 'SAND'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'سندباکس - تومان'
        faBaseAsset = 'سندباکس'
        faQuoteAsset = 'تومان'
        stepSize = 1
        tickSize = 0
        minQty = 0.1
        minNotional = 100000

    class SANDUSDT(_NiceEnum):
        symbol = 'SANDUSDT'
        baseAsset = 'SAND'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'سندباکس - تتر'
        faBaseAsset = 'سندباکس'
        faQuoteAsset = 'تتر'
        stepSize = 1
        tickSize = 3
        minQty = 0.1
        minNotional = 10

    class SANDBTC(_NiceEnum):
        symbol = 'SANDBTC'
        baseAsset = 'SAND'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'سندباکس - بیت کوین'
        faBaseAsset = 'سندباکس'
        faQuoteAsset = 'بیت کوین'
        stepSize = 1
        tickSize = 7
        minQty = 0.1
        minNotional = 0.0001

    class ENJTMN(_NiceEnum):
        symbol = 'ENJTMN'
        baseAsset = 'ENJ'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'انجین - تومان'
        faBaseAsset = 'انجین'
        faQuoteAsset = 'تومان'
        stepSize = 1
        tickSize = 0
        minQty = 0.1
        minNotional = 100000

    class ENJUSDT(_NiceEnum):
        symbol = 'ENJUSDT'
        baseAsset = 'ENJ'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'انجین - تتر'
        faBaseAsset = 'انجین'
        faQuoteAsset = 'تتر'
        stepSize = 1
        tickSize = 3
        minQty = 0.1
        minNotional = 10

    class ENJBTC(_NiceEnum):
        symbol = 'ENJBTC'
        baseAsset = 'ENJ'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'انجین - بیت کوین'
        faBaseAsset = 'انجین'
        faQuoteAsset = 'بیت کوین'
        stepSize = 1
        tickSize = 7
        minQty = 0.1
        minNotional = 0.0001

    class ALICETMN(_NiceEnum):
        symbol = 'ALICETMN'
        baseAsset = 'ALICE'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'آلیس - تومان'
        faBaseAsset = 'آلیس'
        faQuoteAsset = 'تومان'
        stepSize = 2
        tickSize = 0
        minQty = 0.01
        minNotional = 100000

    class ALICEUSDT(_NiceEnum):
        symbol = 'ALICEUSDT'
        baseAsset = 'ALICE'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'آلیس - تتر'
        faBaseAsset = 'آلیس'
        faQuoteAsset = 'تتر'
        stepSize = 2
        tickSize = 2
        minQty = 0.01
        minNotional = 10

    class ALICEBTC(_NiceEnum):
        symbol = 'ALICEBTC'
        baseAsset = 'ALICE'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'آلیس - بیت کوین'
        faBaseAsset = 'آلیس'
        faQuoteAsset = 'بیت کوین'
        stepSize = 2
        tickSize = 6
        minQty = 0.01
        minNotional = 0.0001

    class EGLDTMN(_NiceEnum):
        symbol = 'EGLDTMN'
        baseAsset = 'EGLD'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'الروند - تومان'
        faBaseAsset = 'الروند'
        faQuoteAsset = 'تومان'
        stepSize = 5
        tickSize = 0
        minQty = 1e-05
        minNotional = 100000

    class EGLDUSDT(_NiceEnum):
        symbol = 'EGLDUSDT'
        baseAsset = 'EGLD'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'الروند - تتر'
        faBaseAsset = 'الروند'
        faQuoteAsset = 'تتر'
        stepSize = 5
        tickSize = 2
        minQty = 1e-05
        minNotional = 10

    class EGLDBTC(_NiceEnum):
        symbol = 'EGLDBTC'
        baseAsset = 'EGLD'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'الروند - بیت کوین'
        faBaseAsset = 'الروند'
        faQuoteAsset = 'بیت کوین'
        stepSize = 2
        tickSize = 6
        minQty = 0.01
        minNotional = 0.0001

    class AVAXTMN(_NiceEnum):
        symbol = 'AVAXTMN'
        baseAsset = 'AVAX'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'اوالانچ - تومان'
        faBaseAsset = 'اوالانچ'
        faQuoteAsset = 'تومان'
        stepSize = 5
        tickSize = 0
        minQty = 1e-05
        minNotional = 100000

    class AVAXUSDT(_NiceEnum):
        symbol = 'AVAXUSDT'
        baseAsset = 'AVAX'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'اوالانچ - تتر'
        faBaseAsset = 'اوالانچ'
        faQuoteAsset = 'تتر'
        stepSize = 5
        tickSize = 2
        minQty = 1e-05
        minNotional = 10

    class AVAXBTC(_NiceEnum):
        symbol = 'AVAXBTC'
        baseAsset = 'AVAX'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'اوالانچ - بیت کوین'
        faBaseAsset = 'اوالانچ'
        faQuoteAsset = 'بیت کوین'
        stepSize = 2
        tickSize = 6
        minQty = 0.01
        minNotional = 0.0001

    class NEARTMN(_NiceEnum):
        symbol = 'NEARTMN'
        baseAsset = 'NEAR'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'نیر - تومان'
        faBaseAsset = 'نیر'
        faQuoteAsset = 'تومان'
        stepSize = 2
        tickSize = 0
        minQty = 0.01
        minNotional = 100000

    class NEARUSDT(_NiceEnum):
        symbol = 'NEARUSDT'
        baseAsset = 'NEAR'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'نیر - تتر'
        faBaseAsset = 'نیر'
        faQuoteAsset = 'تتر'
        stepSize = 2
        tickSize = 2
        minQty = 0.01
        minNotional = 10

    class NEARBTC(_NiceEnum):
        symbol = 'NEARBTC'
        baseAsset = 'NEAR'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'نیر - بیت کوین'
        faBaseAsset = 'نیر'
        faQuoteAsset = 'بیت کوین'
        stepSize = 2
        tickSize = 6
        minQty = 0.01
        minNotional = 0.0001

    class XTZTMN(_NiceEnum):
        symbol = 'XTZTMN'
        baseAsset = 'XTZ'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'تزوس - تومان'
        faBaseAsset = 'تزوس'
        faQuoteAsset = 'تومان'
        stepSize = 1
        tickSize = 0
        minQty = 0.1
        minNotional = 100000

    class XTZUSDT(_NiceEnum):
        symbol = 'XTZUSDT'
        baseAsset = 'XTZ'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'تزوس - تتر'
        faBaseAsset = 'تزوس'
        faQuoteAsset = 'تتر'
        stepSize = 1
        tickSize = 3
        minQty = 0.1
        minNotional = 10

    class XTZBTC(_NiceEnum):
        symbol = 'XTZBTC'
        baseAsset = 'XTZ'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'تزوس - بیت کوین'
        faBaseAsset = 'تزوس'
        faQuoteAsset = 'بیت کوین'
        stepSize = 1
        tickSize = 7
        minQty = 0.1
        minNotional = 0.0001

    class SOLTMN(_NiceEnum):
        symbol = 'SOLTMN'
        baseAsset = 'SOL'
        baseAssetPrecision = 8
        quoteAsset = 'TMN'
        quotePrecision = 0
        faName = 'سولانا - تومان'
        faBaseAsset = 'سولانا'
        faQuoteAsset = 'تومان'
        stepSize = 5
        tickSize = 0
        minQty = 1e-05
        minNotional = 100000

    class SOLUSDT(_NiceEnum):
        symbol = 'SOLUSDT'
        baseAsset = 'SOL'
        baseAssetPrecision = 8
        quoteAsset = 'USDT'
        quotePrecision = 8
        faName = 'سولانا - تتر'
        faBaseAsset = 'سولانا'
        faQuoteAsset = 'تتر'
        stepSize = 5
        tickSize = 2
        minQty = 1e-05
        minNotional = 10

    class SOLBTC(_NiceEnum):
        symbol = 'SOLBTC'
        baseAsset = 'SOL'
        baseAssetPrecision = 8
        quoteAsset = 'BTC'
        quotePrecision = 8
        faName = 'سولانا - بیت کوین'
        faBaseAsset = 'سولانا'
        faQuoteAsset = 'بیت کوین'
        stepSize = 2
        tickSize = 6
        minQty = 0.01
        minNotional = 0.0001



from .main import Juan
from .orders_db_wrapper import JuanORM

from .const import COINS
from .const import TYPES
from .const import STATUS
from .const import CURRENCIES


__all__ = ['Juan', 'JuanORM']

__version__ = '0.1.2'
__author__ = 'AMiWR'



import typing as t
from datetime import datetime
import pytz


from .const import COINS, TYPES
from .orders_db_wrapper import JuanORM


# TODO: Sanity check. Not the code, but the code writer ...


class Juan:
    def __init__(self, bot: str, db_url: str):
        self._orm = JuanORM(db_url)

        self._bot: str = bot

    @staticmethod
    def _get_src_dst(symbol: str) -> t.Tuple[str, str]:
        symbol = symbol.upper()
        if symbol.endswith('BTC'):
            return symbol.split('BTC')[0], 'BTC'
        if symbol.endswith('USDT'):
            return symbol.split('USDT')[0], 'USDT'
        if symbol.endswith('TMN'):
            return symbol.split('TMN')[0], 'TMN'

        raise ValueError(f'Invalid symbol: {symbol}')

    def create_order(
            self, store: str, symbol: str, side: str, amount: float,
            price: float, order_type: str,
            reason_information: t.Dict[str, t.Any] = None, deal_groups_id: int = None,
            order_id: str = None, cancel_at: datetime = None
    ) -> JuanORM.Deals:
        src, dst = self._get_src_dst(symbol)

        if side.lower() == 'buy':
            src, dst = dst, src

            estimated_source_amount = amount * price
            estimated_destination_amount = amount

        else:
            estimated_source_amount = amount
            estimated_destination_amount = amount * price

        src = COINS.get(src)
        dst = COINS.get(dst)

        order_type = TYPES.get(order_type.upper())

        if deal_groups_id is None:
            deal_groups_id = self._orm.deal_group__insert(reason_information).id

        _ = self._orm.deals__insert(
            deal_group_id=deal_groups_id,
            store=store,
            source_currency=src,
            destination_currency=dst,
            status='CREATED',
            type_=order_type,
            estimated_source_amount=estimated_source_amount,
            estimated_destination_amount=estimated_destination_amount,
            store_order_id=order_id,
            should_cancel_at=cancel_at,
            bot=self._bot,
        )

        return _

    def cancel_order(self, db_id: int = None, store_order_id: str = None) -> JuanORM.Deals:
        if not (db_id or store_order_id):
            raise ValueError('Either db_id or store_order_id must be provided')
        return self._orm.deals__update(db_id, store_order_id=store_order_id, should_cancel_at=datetime.now(
            pytz.timezone('Asia/Tehran')
        ))

    def open_orders(self, type_: str = None, side: str = None) -> t.List[JuanORM.Deals]:
        if side is not None:
            raise NotImplementedError('Side filtering is not implemented')

        return self._orm.deals__get(bot=self._bot, type_=type_)

    def order_status(self, db_id: int = None, store_order_id: t.Any = None) -> t.List[JuanORM.Deals]:
        if not (db_id or store_order_id):
            raise ValueError('Either db_id or store_order_id must be provided')

        return self._orm.deals__get(id=db_id, store_order_id=store_order_id)

    def create_deal_group(self, reason_information: t.Dict[str, t.Any]) -> JuanORM.DealGroups:
        return self._orm.deal_group__insert(reason_information)



import typing as t
import requests
import aiohttp
import asyncio

from logging import Logger

from .base import BaseClient
from .exceptions import RequestException, APIException

__all__ = [
    'Client',
    'AsyncClient'
]


class Client(BaseClient):
    def __init__(
            self, bot: str, requests_params: t.Optional[t.Dict[str, t.Any]] = None,
            logger: t.Optional[Logger] = Logger("juandb.client.base")
    ):
        super().__init__(bot, requests_params, logger)

    def _init_session(self) -> requests.Session:

        headers = self._get_headers()

        session = requests.session()
        session.headers.update(headers)
        return session

    def _request(self, method, uri: str, signed: bool, **kwargs):

        kwargs = self._get_request_kwargs(method, signed, **kwargs)
        print(f"{method=} ||| {uri=} ||| {kwargs=}")

        self.response = getattr(self.session, method)(uri, **kwargs)
        return self._handle_response(self.response)

    @staticmethod
    def _handle_response(response: requests.Response):
        if not (200 <= response.status_code < 300):
            raise APIException(response, response.status_code, response.text)
        try:
            return response.json()
        except ValueError:
            raise RequestException('Invalid Response: %s' % response.text)

    def _request_api(
            self, method, path: str, signed: bool = False, **kwargs
    ):
        uri = self._create_api_uri(path)
        return self._request(method, uri, signed, **kwargs)

    def _get(self, path, signed=False, **kwargs) -> t.Dict:
        return self._request_api('get', path, signed, **kwargs)

    def _post(self, path, signed=False, **kwargs) -> t.Dict:
        return self._request_api('post', path, signed, **kwargs)

    def _put(self, path, signed=False, **kwargs) -> t.Dict:
        return self._request_api('put', path, signed, **kwargs)

    def _delete(self, path, signed=False, **kwargs) -> t.Dict:
        return self._request_api('delete', path, signed, **kwargs)

    def create_order(
            self,
            store: str,
            symbol: str,
            side: str,
            type: str,
            deal_groups_id: str,
            quantity: float,
            information: t.Dict,
            price: float = None,
            client_id: str = None,
            cancel_at: str = None,
    ) -> t.Dict:
        data = self._prepare_order(symbol=symbol, side=side, order_type=type, quantity=quantity, price=price)

        _ = self.OrderData(
            deal_group_id=deal_groups_id,
            source_currency=data.src,
            destination_currency=data.dst,
            type=data.type,
            estimated_source_amount=data.estimated_source_amount,
            estimated_destination_amount=data.estimated_destination_amount,
            # store_order_id=client_id,
            should_cancel_at=cancel_at,
            bot=self.BOT,
            information=information,
        ).dict()

        return self._post('deals', signed=False, json=_, headers={'x-storeClient': store})

    def order_market(
            self,
            store: str,
            symbol: str,
            side: str,
            deal_groups_id: str,
            quantity: float,
            information: t.Dict,
            price: float = None,
            client_id: str = None,
            cancel_at: str = None,
    ) -> t.Dict:
        return self.create_order(
            store=store,
            symbol=symbol,
            type=self.ORDER_TYPE_MARKET,
            side=side,
            deal_groups_id=deal_groups_id,
            quantity=quantity,
            information=information,
            price=price,
            client_id=client_id,
            cancel_at=cancel_at,
        )

    def order_limit(
            self,
            store: str,
            symbol: str,
            side: str,
            deal_groups_id: str,
            quantity: float,
            information: t.Dict,
            price: float = None,
            client_id: str = None,
            cancel_at: str = None,
    ) -> t.Dict:
        return self.create_order(
            store=store,
            symbol=symbol,
            type=self.ORDER_TYPE_LIMIT,
            side=side,
            deal_groups_id=deal_groups_id,
            quantity=quantity,
            information=information,
            price=price,
            client_id=client_id,
            cancel_at=cancel_at,
        )

    def order_market_buy(
            self,
            store: str,
            symbol: str,
            deal_groups_id: str,
            quantity: float,
            information: t.Dict,
            price: float = None,
            client_id: str = None,
            cancel_at: str = None,
    ) -> t.Dict:
        return self.order_market(
            store=store,
            symbol=symbol,
            side=self.SIDE_BUY,
            deal_groups_id=deal_groups_id,
            quantity=quantity,
            information=information,
            price=price,
            client_id=client_id,
            cancel_at=cancel_at,
        )

    def order_market_sell(
            self,
            store: str,
            symbol: str,
            deal_groups_id: str,
            quantity: float,
            information: t.Dict,
            price: float = None,
            client_id: str = None,
            cancel_at: str = None,
    ) -> t.Dict:
        return self.order_market(
            store=store,
            symbol=symbol,
            side=self.SIDE_SELL,
            deal_groups_id=deal_groups_id,
            quantity=quantity,
            information=information,
            price=price,
            client_id=client_id,
            cancel_at=cancel_at,
        )

    def order_limit_buy(
            self,
            store: str,
            symbol: str,
            deal_groups_id: str,
            quantity: float,
            information: t.Dict,
            price: float = None,
            client_id: str = None,
            cancel_at: str = None,
    ) -> t.Dict:
        return self.order_limit(
            store=store,
            symbol=symbol,
            side=self.SIDE_BUY,
            deal_groups_id=deal_groups_id,
            quantity=quantity,
            information=information,
            price=price,
            client_id=client_id,
            cancel_at=cancel_at,
        )

    def order_limit_sell(
            self,
            store: str,
            symbol: str,
            deal_groups_id: str,
            quantity: float,
            information: t.Dict,
            price: float = None,
            client_id: str = None,
            cancel_at: str = None,
    ) -> t.Dict:
        return self.order_limit(
            store=store,
            symbol=symbol,
            side=self.SIDE_SELL,
            deal_groups_id=deal_groups_id,
            quantity=quantity,
            information=information,
            price=price,
            client_id=client_id,
            cancel_at=cancel_at,
        )

    def cancel_order(self, order_id: str, store: str) -> t.Dict:
        return self._delete(f'deals/{order_id}', signed=False, headers={'x-storeClient': store})

    def get_open_orders(self, store: str, symbol: str = None, side: str = None) -> t.Dict:
        result = self._get('deals/openDeals', signed=False, headers={'x-storeClient': store})

        if symbol is not None:
            # data = self._pick(result.get('result').get('orders'), "symbol", symbol)
            # result['result']['orders'] = data
            raise NotImplementedError('Symbol filtering is not implemented yet')

        if side is not None:
            raise NotImplementedError('Side filtering is not implemented yet')
            # data = self._pick(result.get('result').get('orders'), "side", side)
            # result['result']['orders'] = data

        return result

    def get_order_status(self, order_id: str, store: str) -> t.Dict:
        return self._get(f'deals/{order_id}', signed=False, headers={'x-storeClient': store})

    def close_connection(self):
        if self.session:
            self.session.close()

    def __del__(self):
        self.close_connection()


class AsyncClient(BaseClient):
    def __init__(
            self,
            bot: str,
            requests_params: t.Optional[t.Dict[str, t.Any]] = None,
            loop: t.Optional[asyncio.AbstractEventLoop] = None,
            logger: t.Optional[Logger] = Logger("juandb.client.base")
    ):

        self.loop = loop or asyncio.get_event_loop()
        super().__init__(bot, requests_params, logger)

    @classmethod
    async def create(
            cls,
            bot: str,
            requests_params: t.Optional[t.Dict[str, t.Any]] = None,
            loop: t.Optional[asyncio.AbstractEventLoop] = None,
            logger: t.Optional[Logger] = Logger("juandb.client.base")
    ) -> 'AsyncClient':

        return cls(bot, requests_params, loop, logger)

    def __aenter__(self):
        return self

    def __aexit__(self, exc_type, exc_val, exc_tb):
        self.close_connection()
        return False

    def _init_session(self) -> aiohttp.ClientSession:
        session = aiohttp.ClientSession(
            loop=self.loop,
            headers=self._get_headers()
        )
        return session

    async def _request(self, method, uri: str, signed: bool, **kwargs):

        kwargs = self._get_request_kwargs(method, signed, **kwargs)

        async with getattr(self.session, method)(uri, **kwargs) as response:
            self.response = response
            return await self._handle_response(response)

    @staticmethod
    async def _handle_response(response: aiohttp.ClientResponse):
        if not str(response.status).startswith('2'):
            raise APIException(response, response.status, await response.text())
        try:
            return await response.json()
        except ValueError:
            txt = await response.text()
            raise RequestException(f'Invalid Response: {txt}')

    async def _request_api(self, method, path, signed=False, **kwargs):
        uri = self._create_api_uri(path)
        return await self._request(method, uri, signed, **kwargs)

    async def _get(self, path, signed=False, **kwargs) -> t.Dict:
        return await self._request_api('get', path, signed, **kwargs)

    async def _post(self, path, signed=False, **kwargs) -> t.Dict:
        return await self._request_api('post', path, signed, **kwargs)

    async def _put(self, path, signed=False, **kwargs) -> t.Dict:
        return await self._request_api('put', path, signed, **kwargs)

    async def _delete(self, path, signed=False, **kwargs) -> t.Dict:
        return await self._request_api('delete', path, signed, **kwargs)

    async def create_order(
            self,
            store: str,
            symbol: str,
            side: str,
            type: str,
            deal_groups_id: str,
            information: t.Dict,
            quantity: float,
            price: float = None,
            client_id: str = None,
            cancel_at: str = None,
    ) -> t.Dict:
        data = self._prepare_order(symbol=symbol, side=side, order_type=type, quantity=quantity, price=price)

        _ = self.OrderData(
            deal_group_id=deal_groups_id,
            source_currency=data.src,
            destination_currency=data.dst,
            type=data.type,
            estimated_source_amount=data.estimated_source_amount,
            estimated_destination_amount=data.estimated_destination_amount,
            # store_order_id=client_id,
            should_cancel_at=cancel_at,
            bot=self.BOT,
            information=information,
        ).dict()

        return await self._post('deals', signed=False, json=_, headers={'x-storeClient': store})

    async def order_market(
            self,
            store: str,
            symbol: str,
            side: str,
            deal_groups_id: str,
            quantity: float,
            information: t.Dict,
            price: float = None,
            client_id: str = None,
            cancel_at: str = None,
    ) -> t.Dict:
        return await self.create_order(
            store=store,
            symbol=symbol,
            type=self.ORDER_TYPE_MARKET,
            side=side,
            deal_groups_id=deal_groups_id,
            quantity=quantity,
            information=information,
            price=price,
            client_id=client_id,
            cancel_at=cancel_at,
        )

    async def order_limit(
            self,
            store: str,
            symbol: str,
            side: str,
            deal_groups_id: str,
            quantity: float,
            information: t.Dict,
            price: float = None,
            client_id: str = None,
            cancel_at: str = None,
    ) -> t.Dict:
        return await self.create_order(
            store=store,
            symbol=symbol,
            type=self.ORDER_TYPE_LIMIT,
            side=side,
            deal_groups_id=deal_groups_id,
            quantity=quantity,
            information=information,
            price=price,
            client_id=client_id,
            cancel_at=cancel_at,
        )

    async def order_market_buy(
            self,
            store: str,
            symbol: str,
            deal_groups_id: str,
            quantity: float,
            information: t.Dict,
            price: float = None,
            client_id: str = None,
            cancel_at: str = None,
    ) -> t.Dict:
        return await self.order_market(
            store=store,
            symbol=symbol,
            side=self.SIDE_BUY,
            deal_groups_id=deal_groups_id,
            quantity=quantity,
            information=information,
            price=price,
            client_id=client_id,
            cancel_at=cancel_at,
        )

    async def order_market_sell(
            self,
            store: str,
            symbol: str,
            deal_groups_id: str,
            quantity: float,
            information: t.Dict,
            price: float = None,
            client_id: str = None,
            cancel_at: str = None,
    ) -> t.Dict:
        return await self.order_market(
            store=store,
            symbol=symbol,
            side=self.SIDE_SELL,
            deal_groups_id=deal_groups_id,
            quantity=quantity,
            information=information,
            price=price,
            client_id=client_id,
            cancel_at=cancel_at,
        )

    async def order_limit_buy(
            self,
            store: str,
            symbol: str,
            deal_groups_id: str,
            quantity: float,
            information: t.Dict,
            price: float = None,
            client_id: str = None,
            cancel_at: str = None,
    ) -> t.Dict:
        return await self.order_limit(
            store=store,
            symbol=symbol,
            side=self.SIDE_BUY,
            deal_groups_id=deal_groups_id,
            quantity=quantity,
            information=information,
            price=price,
            client_id=client_id,
            cancel_at=cancel_at,
        )

    async def order_limit_sell(
            self,
            store: str,
            symbol: str,
            deal_groups_id: str,
            quantity: float,
            information: t.Dict,
            price: float = None,
            client_id: str = None,
            cancel_at: str = None,
    ) -> t.Dict:
        return await self.order_limit(
            store=store,
            symbol=symbol,
            side=self.SIDE_SELL,
            deal_groups_id=deal_groups_id,
            quantity=quantity,
            information=information,
            price=price,
            client_id=client_id,
            cancel_at=cancel_at,
        )

    async def cancel_order(self, order_id: str, store: str) -> t.Dict:
        return await self._delete(f'deals/{order_id}', signed=False, headers={'x-storeClient': store})

    async def get_open_orders(self, store: str, symbol: str = None, side: str = None) -> t.Dict:
        result = await self._get('deals/openDeals', signed=False, headers={'x-storeClient': store})

        if symbol is not None:
            data = self._pick(result.get('result').get('orders'), "symbol", symbol)
            result['result']['orders'] = data

        if side is not None:
            data = self._pick(result.get('result').get('orders'), "side", side)
            result['result']['orders'] = data

        return result

    async def get_order_status(self, order_id: str, store: str) -> t.Dict:
        return await self._get(f'deals/{order_id}', signed=False, headers={'x-storeClient': store})

    async def close_connection(self):
        if self.session:
            assert self.session
            await self.session.close()

    def __del__(self):
        asyncio.get_event_loop().create_task(self.close_connection())



from enum import Enum


__all__ = [
    'COINS',
    'TYPES',
    'STATUS'
]


class _Enum(Enum):
    def __str__(self):
        return self.value

    def __repr__(self):
        return self.value

    @classmethod
    def dict(cls):
        return {str(e.value): e.value for e in cls}

    @classmethod
    def get(cls, key: str = None):
        if key is None:
            return cls.dict()
        return cls.dict().get(key)


class COINS(_Enum):
    BITCOIN = 'BITCOIN'
    TETHER = 'TETHER'
    ETHEREUM = 'ETHEREUM'
    TOMAN = 'TOMAN'
    RIPPLE = 'RIPPLE'
    BITCOIN_CASH = 'BITCOIN_CASH'
    LITE_COIN = 'LITE_COIN'
    EOS = 'EOS'
    STELLAR = 'STELLAR'
    TRON = 'TRON'
    PAXG = 'PAXG'
    DASH = 'DASH'
    BINANCE_COIN = 'BINANCE_COIN'
    DOGE = 'DOGE'
    CARDANO = 'CARDANO'
    ATOM = 'ATOM'
    MATIC = 'MATIC'
    FANTOM = 'FANTOM'
    POLKADOT = 'POLKADOT'
    SHIBA = 'SHIBA'
    FIL = 'FIL'
    CAKE = 'CAKE'
    CHAIN_LINK = 'CHAIN_LINK'
    UNISWAP = 'UNISWAP'
    THORCHAIN = 'THORCHAIN'
    CHILIZ = 'CHILIZ'
    BIT_TORRENT = 'BIT_TORRENT'
    DECENTRALAND = 'DECENTRALAND'
    AXIE_INFINITY = 'AXIE_INFINITY'
    SANDBOX = 'SANDBOX'
    ENJIN_COIN = 'ENJIN_COIN'
    MY_NEIGHBOR_ALICE = 'MY_NEIGHBOR_ALICE'
    ELROND_EGOLD = 'ELROND_EGOLD'
    AVALANCHE = 'AVALANCHE'
    NEAR_PROTOCOL = 'NEAR_PROTOCOL'
    SOLANA = 'SOLANA'
    TEZOS = 'TEZOS'
    DAI = 'DAI'
    ETHEREUM_CLASSIC = 'ETHEREUM_CLASSIC'
    GALA = 'GALA'
    AAVE = 'AAVE'
    MAKER = 'MAKER'
    ONE_INCH_NETWORK = 'ONE_INCH_NETWORK'
    YFI = 'YFI'
    CELR = 'CELR'
    APE = 'APE'
    POLKASTARTER = 'POLKASTARTER'

    @classmethod
    def dict(cls):
        return {
            'BTC': cls.BITCOIN.name,
            'USDT': cls.TETHER.name,
            'ETH': cls.ETHEREUM.name,
            'TMN': cls.TOMAN.name,
            'XRP': cls.RIPPLE.name,
            'BCH': cls.BITCOIN_CASH.name,
            'LTC': cls.LITE_COIN.name,
            'EOS': cls.EOS.name,
            'XLM': cls.STELLAR.name,
            'TRX': cls.TRON.name,
            'PAXG': cls.PAXG.name,
            'DASH': cls.DASH.name,
            'BNB': cls.BINANCE_COIN.name,
            'DOGE': cls.DOGE.name,
            'ADA': cls.CARDANO.name,
            'ATOM': cls.ATOM.name,
            'MATIC': cls.MATIC.name,
            'FTM': cls.FANTOM.name,
            'DOT': cls.POLKADOT.name,
            'SHIB': cls.SHIBA.name,
            'FIL': cls.FIL.name,
            'CAKE': cls.CAKE.name,
            'LINK': cls.CHAIN_LINK.name,
            'UNI': cls.UNISWAP.name,
            'RUNE': cls.THORCHAIN.name,
            'CHZ': cls.CHILIZ.name,
            'BTT': cls.BIT_TORRENT.name,
            'MANA': cls.DECENTRALAND.name,
            'AXS': cls.AXIE_INFINITY.name,
            'SAND': cls.SANDBOX.name,
            'ENJ': cls.ENJIN_COIN.name,
            'ALICE': cls.MY_NEIGHBOR_ALICE.name,
            'EGLD': cls.ELROND_EGOLD.name,
            'AVAX': cls.AVALANCHE.name,
            'NEAR': cls.NEAR_PROTOCOL.name,
            'SOL': cls.SOLANA.name,
            'XTZ': cls.TEZOS.name,
            'DAI': cls.DAI.name,
            'ETC': cls.ETHEREUM_CLASSIC.name,
            'GALA': cls.GALA.name,
            'AAVE': cls.AAVE.name,
            'MAKER': cls.MAKER.name,
            '1INCH': cls.ONE_INCH_NETWORK.name,
            'YFI': cls.YFI.name,
            'CELR': cls.CELR.name,
            'APE': cls.APE.name,
            'POLS': cls.POLKASTARTER.name
        }


class TYPES(_Enum):
    QUICK = 'QUICK'
    NORMAL = 'NORMAL'

    @classmethod
    def dict(cls):
        return {
            'LIMIT': cls.NORMAL.value,
            'MARKET': cls.QUICK.value
        }


class STATUS(_Enum):
    CREATED = 'CREATED'
    ORDERING = 'ORDERING'
    ORDERED = 'ORDERED'

    FAILED = 'FAILED'
    FINISHED = 'FINISHED'
    CANCELING = 'CANCELING'
    CANCELED = 'CANCELED'



import typing as t

from sqlalchemy import create_engine
from sqlalchemy import Column, String, BigInteger, DateTime, ForeignKey, JSON, DECIMAL, Integer, Boolean, text
from sqlalchemy.orm import sessionmaker, Query
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.exc import PendingRollbackError, IntegrityError

from datetime import datetime
import pytz

import json


def _get_kwargs(params: t.Dict[str, t.Any]) -> t.Dict[str, t.Any]:
    try:
        params.pop('self')
    except KeyError:
        pass
    return {k: v for k, v in params.items() if v is not None}


class JuanORM:
    __BASE = declarative_base()

    def __init__(self, db_url: str, create_tables: bool = False):
        self._engine = create_engine(db_url)
        self._engine.connect()

        self._Base = self.__BASE
        self._Base.bind = self._engine

        self._Session = sessionmaker(bind=self._engine)
        self._session = self._Session()

        if create_tables is True:
            self.create_tables()

    @staticmethod
    def serialize(result: t.List) -> t.List[t.Dict[str, t.Any]]:
        return [_.dict() for _ in result]

    def _add(self, obj: object):
        try:
            self._session.add(obj)
        except PendingRollbackError:
            self._rollback()
        except Exception as e:
            print(e)
            self._session.rollback()

    def _commit(self):
        try:
            self._session.commit()
        except PendingRollbackError:
            self._rollback()
        except IntegrityError:
            self._session.rollback()
        except Exception as e:
            print(e)
            self._session.rollback()

    def _rollback(self):
        self._session.rollback()

    def _query(self, table) -> Query:
        try:
            self._session.commit()
        except PendingRollbackError:
            self._rollback()
        except Exception as e:
            self._rollback()
            raise e

        try:
            return self._session.query(table)
        except Exception as e:
            print(e)
            self._rollback()

    def _add_commit(self, obj: object):
        self._add(obj)
        self._commit()

    def _execute(self, sql: str):
        try:
            self._session.execute(sql)
        except PendingRollbackError:
            self._rollback()
        except Exception as e:
            print(e)
            self._session.rollback()

    def _execute_sql(self, sql: str):
        self._execute(sql)
        self._commit()

    def create_tables(self):
        self._Base.metadata.create_all(self._engine)

    def drop_table(self, table: str):
        self._Base.metadata.tables[table].drop(self._engine)

    def drop_all_tables(self):
        self._Base.metadata.drop_all(self._engine)

    def recreate_all_tables(self):
        self.drop_all_tables()
        self.create_tables()

    class DealGroups(__BASE):
        __tablename__ = 'deal_groups'

        id = Column(BigInteger, primary_key=True, autoincrement=True)
        reason_information = Column(JSON, nullable=True)
        created_at = Column(DateTime, nullable=True, default=None)
        updated_at = Column(DateTime, nullable=True, default=None)

        def __repr__(self):
            return f'<DealGroups(id={self.id}, ' \
                   f'reason_information={self.reason_information}, ' \
                   f'created_at={self.created_at}, ' \
                   f'updated_at={self.updated_at})>'

        def __str__(self):
            return str(self.__repr__())

        def dict(self) -> t.Dict[str, t.Union[str, t.Dict, datetime]]:
            return {
                'id': self.id,
                'reason_information': self.reason_information,
                'created_at': self.created_at,
                'updated_at': self.updated_at
            }

        def json(self):
            return json.dumps(self.dict())

    def deal_group__get(
            self, id: int, reason_information: t.Dict = None, created_at: str = None, updated_at: str = None
    ) -> t.List[DealGroups]:
        kwargs: t.Dict[str, t.Any] = _get_kwargs(locals())

        # get self.Deals with given parameters
        data = self._query(self.DealGroups).filter_by(**kwargs).all()
        return data

    def deal_group__insert(
            self, reason_information: t.Dict = None, created_at: str = None, updated_at: str = None
    ) -> DealGroups:

        if created_at is None:
            created_at = datetime.now(tz=pytz.timezone('Asia/Tehran'))
        if updated_at is None:
            updated_at = datetime.now(tz=pytz.timezone('Asia/Tehran'))

        _ = self.DealGroups(
            reason_information=reason_information,
            created_at=created_at,
            updated_at=updated_at
        )
        self._add_commit(_)
        return _

    def deal_group__update(
            self, id: int, reason_information: str = None, created_at: str = None, updated_at: str = None
    ) -> DealGroups:
        kwargs = _get_kwargs(locals())

        if len(kwargs) == 0:
            raise ValueError('Nothing to update')

        kwargs['updated_at'] = datetime.now(tz=pytz.timezone('Asia/Tehran'))

        _ = self.DealGroups.__table__.update().where(self.DealGroups.id == id).values(**kwargs)
        self._add_commit(_)

        # _ = self._query(self.DealGroups).filter(self.DealGroups.id == id).update(d)
        self._commit()

        return _

    def deal_group__delete(self, id: int):
        self._query(self.DealGroups).filter(self.DealGroups.id == id).delete()
        self._commit()

    class Deals(__BASE):
        __tablename__ = 'deals'

        id = Column(BigInteger, primary_key=True, autoincrement=True)
        deal_group_id = Column(BigInteger, ForeignKey('deal_groups.id'), nullable=False)
        store = Column(String(255), nullable=False)
        source_currency = Column(String(255), nullable=False)
        destination_currency = Column(String(255), nullable=False)
        estimated_source_amount = Column(DECIMAL(30, 15), nullable=True, default=None)
        processed_source_amount = Column(DECIMAL(30, 15), nullable=True, default=None)
        estimated_destination_amount = Column(DECIMAL(30, 15), nullable=True, default=None)
        processed_destination_amount = Column(DECIMAL(30, 15), nullable=True, default=None)
        status = Column(String(255), nullable=False)
        type = Column(String(255), nullable=False)
        processed_at = Column(DateTime, nullable=True, default=None)
        should_cancel_at = Column(DateTime, nullable=True, default=None)
        finalized_at = Column(DateTime, nullable=True, default=None)
        created_at = Column(DateTime, nullable=True, default=None)
        updated_at = Column(DateTime, nullable=True, default=None)
        store_order_id = Column(String(255), nullable=True, default=None)
        logs = Column(JSON, nullable=True, default=None)
        bot = Column(String(255), nullable=True, default=None)
        owner_type = Column(String(255), nullable=True, default=None)
        owner_id = Column(BigInteger, nullable=True, default=None)

        def __repr__(self):
            return f'<Deal(id={self.id}, ' \
                   f'deal_group_id={self.deal_group_id}, ' \
                   f'store={self.store}, ' \
                   f'source_currency={self.source_currency}, ' \
                   f'destination_currency={self.destination_currency}, ' \
                   f'estimated_source_amount={self.estimated_source_amount}, ' \
                   f'processed_source_amount={self.processed_source_amount}, ' \
                   f'estimated_destination_amount={self.estimated_destination_amount}, ' \
                   f'processed_destination_amount={self.processed_destination_amount}, ' \
                   f'status={self.status}, ' \
                   f'type={self.type}, ' \
                   f'processed_at={self.processed_at}, ' \
                   f'should_cancel_at={self.should_cancel_at}, ' \
                   f'finalized_at={self.finalized_at}, ' \
                   f'created_at={self.created_at}, ' \
                   f'updated_at={self.updated_at}, ' \
                   f'store_order_id={self.store_order_id}, ' \
                   f'logs={self.logs}, ' \
                   f'bot={self.bot}, ' \
                   f'owner_type={self.owner_type}, ' \
                   f'owner_id={self.owner_id})>'

        def __str__(self):
            return str(self.__repr__())

        def dict(self):
            return {
                'id': self.id,
                'deal_group_id': self.deal_group_id,
                'store': self.store,
                'source_currency': self.source_currency,
                'destination_currency': self.destination_currency,
                'estimated_source_amount': self.estimated_source_amount,
                'processed_source_amount': self.processed_source_amount,
                'estimated_destination_amount': self.estimated_destination_amount,
                'processed_destination_amount': self.processed_destination_amount,
                'status': self.status,
                'type': self.type,
                'processed_at': self.processed_at,
                'should_cancel_at': self.should_cancel_at,
                'finalized_at': self.finalized_at,
                'created_at': self.created_at,
                'updated_at': self.updated_at,
                'store_order_id': self.store_order_id,
                'logs': self.logs,
                'bot': self.bot,
                'owner_type': self.owner_type,
                'owner_id': self.owner_id
            }

        def json(self):
            return json.dumps(self.dict())

    def deals__get(
            self,
            id: int = None,
            deal_group_id: int = None,
            store: str = None,
            source_currency: str = None,
            destination_currency: str = None,
            status: str = None,
            type_: str = None,
            estimated_source_amount: float = None,
            processed_source_amount: float = None,
            estimated_destination_amount: float = None,
            processed_destination_amount: float = None,
            processed_at: datetime = None,
            should_cancel_at: datetime = None,
            finalized_at: datetime = None,
            created_at: datetime = None,
            updated_at: datetime = None,
            store_order_id: str = None,
            logs: str = None,
            bot: str = None,
            owner_type: str = None,
            owner_id: int = None
    ) -> t.List[Deals]:
        kwargs: t.Dict[str, t.Any] = _get_kwargs(locals())

        # get self.Deals with given parameters
        data = self._query(self.Deals).filter_by(**kwargs).all()
        return data

    def deals__insert(
            self,
            deal_group_id: int,
            store: str,
            source_currency: str,
            destination_currency: str,
            status: str,
            type_: str,
            estimated_source_amount: float = None,
            processed_source_amount: float = None,
            estimated_destination_amount: float = None,
            processed_destination_amount: float = None,
            processed_at: datetime = None,
            should_cancel_at: datetime = None,
            finalized_at: datetime = None,
            created_at: datetime = None,
            updated_at: datetime = None,
            store_order_id: str = None,
            logs: str = None,
            bot: str = None,
            owner_type: str = None,
            owner_id: int = None
    ) -> Deals:
        if created_at is None:
            created_at = datetime.now()
        if updated_at is None:
            updated_at = datetime.now()

        _ = self.Deals(
            deal_group_id=deal_group_id, store=store, source_currency=source_currency,
            destination_currency=destination_currency, estimated_source_amount=estimated_source_amount,
            processed_source_amount=processed_source_amount, estimated_destination_amount=estimated_destination_amount,
            processed_destination_amount=processed_destination_amount, status=status, type=type_,
            processed_at=processed_at, should_cancel_at=should_cancel_at, finalized_at=finalized_at,
            created_at=created_at, updated_at=updated_at, store_order_id=store_order_id,
            logs=logs, bot=bot, owner_type=owner_type, owner_id=owner_id
        )
        self._add_commit(_)
        return _

    def deals__update(
            self,
            id: int = None,
            deal_group_id: int = None,
            store: str = None,
            source_currency: str = None,
            destination_currency: str = None,
            status: str = None,
            type_: str = None,
            estimated_source_amount: float = None,
            processed_source_amount: float = None,
            estimated_destination_amount: float = None,
            processed_destination_amount: float = None,
            processed_at: datetime = None,
            should_cancel_at: datetime = None,
            finalized_at: datetime = None,
            created_at: datetime = None,
            updated_at: datetime = None,
            store_order_id: str = None,
            logs: str = None,
            bot: str = None,
            owner_type: str = None,
            owner_id: int = None
    ):
        kwargs = _get_kwargs(locals())
        if len(kwargs) == 0:
            raise ValueError('Nothing to update')

        kwargs['updated_at'] = datetime.utcnow()

        if 'id' in kwargs.keys():
            self._query(self.Deals).filter_by(id=id).update(kwargs)
        elif 'store_order_id' in kwargs.keys():
            self._query(self.Deals).filter_by(store_order_id=store_order_id).update(kwargs)
        else:
            raise ValueError('Either id or store_order_id must be specified')

        self._commit()

    def deals__delete(self, id: int):
        self._query(self.Deals).filter_by(id=id).delete()
        self._commit()

    class CoinMap(__BASE):
        __tablename__ = 'coin_map'
        id = Column(Integer, primary_key=True, autoincrement=True, nullable=False, unique=True, index=True, doc='ID')
        symbol = Column(String(255), nullable=False, unique=True)
        name = Column(String(255), nullable=False, unique=True)
        is_active = Column(Boolean, nullable=False, default=True, server_default=text('true'))
        is_fiat = Column(Boolean, nullable=False, default=False, server_default=text('false'))
        is_crypto = Column(Boolean, nullable=False, default=True, server_default=text('true'))
        created_at = Column(DateTime, nullable=False, server_default=text('now()'))
        updated_at = Column(DateTime, nullable=False, server_default=text('now()'), onupdate=text('now()'))

        def __repr__(self):
            return f'<CoinMap(id={self.id}, ' \
                   f'symbol={self.symbol}, ' \
                   f'name={self.name}, ' \
                   f'is_active={self.is_active}, ' \
                   f'is_fiat={self.is_fiat}, ' \
                   f'is_crypto={self.is_crypto}, ' \
                   f'created_at={self.created_at}, ' \
                   f'updated_at={self.updated_at})>'

        def __str__(self):
            return self.__repr__()

        def dict(self):
            return {
                'id': self.id,
                'symbol': self.symbol,
                'name': self.name,
                'is_active': self.is_active,
                'is_fiat': self.is_fiat,
                'is_crypto': self.is_crypto,
                'created_at': self.created_at,
                'updated_at': self.updated_at
            }

        def json(self):
            return json.dumps(self.dict())

    def coin_map__insert(
            self, symbol: str, name: str, is_active: bool = True, is_fiat: bool = False,
            is_crypto: bool = True
    ) -> CoinMap:
        _ = self.CoinMap(
            symbol=symbol,
            name=name,
            is_active=is_active,
            is_fiat=is_fiat,
            is_crypto=is_crypto,
        )
        self._add_commit(_)

        return _

    def coin_map__get(self, id: int = None, symbol: str = None, name: str = None) -> t.List[CoinMap]:
        kwargs: t.Dict[str, t.Any] = _get_kwargs(locals())

        # get self.Deals with given parameters
        data = self._query(self.CoinMap).filter_by(**kwargs).all()
        return data

    def coin_map__map(self):
        data = self.coin_map__get()
        _ = {}

        for coin in data:
            _[coin.symbol] = coin.name

        return _



from distutils.core import setup
from setuptools import find_packages

setup(
    name='juandb',
    version='0.1.2',
    license='MIT',
    description='Nice',
    author='amiwrpremium',
    author_email='amiwrpremium@gmail.com',
    url='https://github.com/amiwrpremium/juan',
    keywords=['NICE', 'JUAN'],
    install_requires=[
        'sqlalchemy',
        'pymysql',
    ],
    package_dir={'': 'src'},
    packages=find_packages(where='src'),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)
